MODULE Hankel_Single

IMPLICIT NONE

PUBLIC :: CHankel

CONTAINS
      SUBROUTINE CHankel(n,h,b,x,nref,factor,ifail)
!-----------------------------------------------------------------------
!***BEGIN PROLOGUE CHankel 
!***DATE WRITTEN   960710    (YYMMDD)
!***AUTHOR
!          Peter Kravanja
!          Dept. Computer Science
!          Katholieke Universiteit Leuven
!          Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
!
!          E-mail: na.pkravanja@na-net.ornl.gov
!
!          Marc Van Barel
!          Dept. Computer Science
!          Katholieke Universiteit Leuven
!          Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
!
!          E-mail: Marc.VanBarel@cs.kuleuven.ac.be 
!
!***PURPOSE
!     To solve linear systems Hx=b with H a complex regular n x n
!     Hankel matrix in O(n^2) flops.
!
!-----------------------------------------------------------------------
!
!   Input Parameters
!   ----------------
!
!   n       Integer
!           The size of the system.
!
!   h       Complex array of dimension (0:2*n-2) 
!           (Matlab notation) H = hankel(h(0:n-1),h(n-1:2*n-2)).
!
!   b       Complex array of dimension (1:n)
!           The right hand side.
!
!   nref    Integer, optional (default nref = 1)
!
!           The number of times that the computed solution is to be
!           improved iteratively. Each step costs O(n log n) flops.
!
!   factor  Real, optional (default factor = 1E+02)
!
!           The larger part of the algorithm is concerned with the
!           solution of a rational interpolation problem on the unit
!           circle (subroutine Ratint).
!           The algorithm stops if all the residues are smaller than
!           factor times the machine epsilon. This means that H is
!           singular or ill-conditioned. In the latter case you may
!           want to lower the value of factor.
!
!   ifail   Integer, optional  
!
!           This follows the NAG convention:
!            ifail = 1 : soft silent error
!                        Control returned to calling program.
!            ifail = -1: soft noisy error
!                        Error message is printed.
!                        Control returned to calling program.
!            ifail = 0 : hard noisy error
!                        Error message is printed and program is stopped.
!           Default ifail = 0.
!
!   Output Parameters
!   -----------------
!
!   x       Complex array of dimension (1:n) 
!           The computed solution to Hx=b.
!
!   ifail   Integer
!            ifail = 0 for normal exit.
!            ifail = 1 if the matrix is (nearly) singular. In this case 
!                      you may want to lower the value of factor.
!
!***END PROLOGUE CHankel 
!-----------------------------------------------------------------------
!
!  Global variables
!
      INTEGER, INTENT(IN)                     :: n
      INTEGER, OPTIONAL, INTENT(IN)           :: nref 
      INTEGER, OPTIONAL, INTENT(INOUT)        :: ifail
      REAL, OPTIONAL, INTENT(IN)              :: factor
      COMPLEX, DIMENSION(n), INTENT(IN)       :: b
      COMPLEX, DIMENSION(0:2*n-2), INTENT(IN) :: h
      COMPLEX, DIMENSION(n), INTENT(OUT)      :: x
!
!  Local variables
!
      LOGICAL                 :: h_is_zero, test
      INTEGER                 :: numref, k, numifail, j, hn
      INTEGER, DIMENSION(15)  :: ifac, ifac2
      INTEGER, DIMENSION(2*n) :: pivot, invpivot
      REAL                    :: pi, numfactor, alpha
      REAL, DIMENSION(2*n)    :: wa, ch
      REAL, DIMENSION(4*n)    :: wa2, ch2
      COMPLEX, PARAMETER      :: I = (0.,1.)
      COMPLEX                 :: zeta
      COMPLEX, DIMENSION(n)   :: a, y, z, zeta_n, c, d, bpr, xpr
      COMPLEX, DIMENSION(n)   :: u, utilde, p, ptilde, aux1, aux2
      COMPLEX, DIMENSION(n)   :: residue, residuepr, xnew, xnewpr
      COMPLEX, DIMENSION(2*n) :: a2, zeta_2n, hvec, param, dl, dr
      COMPLEX, DIMENSION(2*n) :: capz, capf
      DOUBLE COMPLEX, DIMENSION(3*n-2) :: cv
      DOUBLE COMPLEX, DIMENSION(n) :: b2
!
!  External routines
!
      EXTERNAL cffti1, cfftf1, cfftb1
!-----------------------------------------------------------------------
      pi = 4.*ATAN(1.)

      IF (PRESENT(nref)) THEN
        numref = nref
      ELSE
        numref = 1
      END IF 

      IF (PRESENT(factor)) THEN
        numfactor = ABS(factor)
      ELSE
        numfactor = 1E+02
      END IF

      IF (PRESENT(ifail)) THEN
        numifail = ifail
      ELSE
        numifail = 0 
      END IF 

      h_is_zero = .TRUE.
      DO k = 0, 2*n-2
        test = (h(k) == 0.)
        h_is_zero = h_is_zero .AND. test
      END DO

      IF (h_is_zero) THEN
        IF (PRESENT(ifail)) ifail = 1
        SELECT CASE(numifail)
            CASE(1)
              RETURN
            CASE(-1)
              PRINT *; PRINT 6000; PRINT 8001
              RETURN
            CASE(0)
              PRINT *; PRINT 6000; PRINT 8002
              STOP
        END SELECT
      END IF

      IF (n == 1) THEN 
        IF (h(0) == 0) THEN
          IF (PRESENT(ifail)) ifail = 1
          SELECT CASE(numifail)
            CASE(1)
              RETURN 
            CASE(-1)
              PRINT *; PRINT 8000; PRINT 8001
              RETURN
            CASE(0)
              PRINT *; PRINT 8000; PRINT 8002
              STOP
          END SELECT
        ELSE
          IF (PRESENT(ifail)) ifail = 0
          x = b(1)/h(0)
        END IF
      ELSE
        CALL cffti1(n,wa,ifac)
        a = 0.
        a(2) = 1.
        CALL cfftf1(n,a,ch,wa,ifac) 
        y = CONJG(a)
        zeta = EXP(pi*I/n)
        z = zeta*y 

        CALL cffti1(2*n,wa2,ifac2)
        a2 = 0.
        a2(2) = 1.
        CALL cfftf1(2*n,a2,ch2,wa2,ifac2)
        zeta_2n = CONJG(a2)
        zeta_n = zeta_2n(1:n)

        DO k = 1,n
          pivot(2*k-1) = k 
          pivot(2*k) = n+k 
          invpivot(k) = 2*k-2
          invpivot(k+n) = 2*k-1
        END DO
        invpivot = 1 + invpivot

        hvec(1:2*n-1) = h(0:2*n-2)
        hvec(2*n) = 0.
        CALL cfftf1(2*n,hvec,ch2,wa2,ifac2) 
        param = HP(CONJG(zeta_2n),hvec)
        param = Swap(param,invpivot)
        c = param(1:n)
        d = param(n+1:2*n)

        bpr = b
        CALL cfftf1(n,bpr,ch,wa,ifac)
        bpr = HP(CONJG(y),bpr)

        capz(1:n) = y; capz(n+1:2*n) = z
        capf(1:n) = c; capf(n+1:2*n) = d      
        capz = Swap(capz,pivot)
        capf = Swap(capf,pivot)

        CALL Ratint(2*n,capz,capf,numfactor,dl,dr,alpha,hn)

        IF (hn < 2*n) THEN
          IF (PRESENT(ifail)) ifail = 1
          SELECT CASE(numifail)
            CASE(1)
              RETURN
            CASE(-1)
              PRINT *; PRINT 8000; PRINT 8001; PRINT 8003
              RETURN
            CASE(0)
              PRINT *; PRINT 8000; PRINT 8002; PRINT 8003
              STOP
          END SELECT
        ELSE
          IF (PRESENT(ifail)) ifail = 0

          dl = Swap(dl,invpivot)
          dr = Swap(dr,invpivot)
          utilde = -CONJG(zeta**(n-1))*HP(y,dr(n+1:2*n))/n 
          u = -HP(y,dr(1:n))/n
          ptilde = CONJG(zeta**(n-1))*HP(y,dl(n+1:2*n))/n
          p = HP(y,dl(1:n))/n

          aux1 = HP(y,CONJG(HP(u,bpr)))
          CALL cfftb1(n,aux1,ch,wa,ifac)
          aux1 = aux1/n
          aux1 = HP(CONJG(zeta_n),aux1)
          CALL cfftf1(n,aux1,ch,wa,ifac)
          aux1 = HP(ptilde,CONJG(aux1))

          aux2 = HP(y,CONJG(HP(p,bpr)))
          CALL cfftb1(n,aux2,ch,wa,ifac)
          aux2 = aux2/n
          aux2 = HP(CONJG(zeta_n),aux2)
          CALL cfftf1(n,aux2,ch,wa,ifac)
          aux2 = HP(utilde,CONJG(aux2))

          xpr = n*(aux2 - aux1)/2

          aux1 = HP(y,CONJG(xpr))
          CALL cfftb1(n,aux1,ch,wa,ifac)
          aux1 = aux1/n
          x = n*HP(zeta_n(n:1:-1),CONJG(aux1))/alpha

          DO k = 1,numref
            cv = Conv(h(0:2*n-2),x(n:1:-1))
            b2 = b
            residue = b2 - cv(n:2*n-1) 
            CALL cfftf1(n,residue,ch,wa,ifac)
            residuepr = HP(CONJG(y),residue)

            aux1 = HP(y,CONJG(HP(u,residuepr)))
            CALL cfftb1(n,aux1,ch,wa,ifac)
            aux1 = aux1/n
            aux1 = HP(CONJG(zeta_n),aux1)
            CALL cfftf1(n,aux1,ch,wa,ifac)
            aux1 = HP(ptilde,CONJG(aux1))

            aux2 = HP(y,CONJG(HP(p,residuepr)))
            CALL cfftb1(n,aux2,ch,wa,ifac)
            aux2 = aux2/n
            aux2 = HP(CONJG(zeta_n),aux2)
            CALL cfftf1(n,aux2,ch,wa,ifac)
            aux2 = HP(utilde,CONJG(aux2))

            xnewpr = n*(aux2 - aux1)/2

            aux1 = HP(y,CONJG(xnewpr))
            CALL cfftb1(n,aux1,ch,wa,ifac)
            aux1 = aux1/n
            xnew = n*HP(zeta_n(n:1:-1),CONJG(aux1))/alpha

            x = x + xnew
          END DO
        END IF
      END IF

8000  FORMAT(' Matrix is (nearly) singular.')
8001  FORMAT(' Control is returned to calling program.'/)
8002  FORMAT(' Program is stopped.'/)
8003  FORMAT(' You may want to lower the value of factor.'/)

6000  FORMAT(' Matrix is equal to the zero matrix.')

      END SUBROUTINE CHankel
!
!-----------------------------------------------------------------------
!
      FUNCTION HP(a,b)
      COMPLEX, DIMENSION(:) :: a
      COMPLEX, DIMENSION(SIZE(a)) :: b, HP
      INTEGER k
 
      DO k = 1,SIZE(a) 
        HP(k) = a(k) * b(k)
      END DO
      END FUNCTION HP
!
!-----------------------------------------------------------------------
!
      FUNCTION DHP(a,b)
      DOUBLE COMPLEX, DIMENSION(:) :: a
      DOUBLE COMPLEX, DIMENSION(SIZE(a)) :: b, DHP
      INTEGER k

      DO k = 1,SIZE(a)
        DHP(k) = a(k) * b(k)
      END DO
      END FUNCTION DHP
!
!-----------------------------------------------------------------------
!
      FUNCTION Swap(a,pivot)
      COMPLEX, DIMENSION(:) :: a
      INTEGER, DIMENSION(SIZE(a)) :: pivot
      COMPLEX, DIMENSION(SIZE(a)) :: Swap
      INTEGER k

      DO k = 1,SIZE(a)
        Swap(k) = a(pivot(k))
      END DO
      END FUNCTION Swap
!
!-----------------------------------------------------------------------
!
      FUNCTION Conv(a,b)
      COMPLEX, DIMENSION(:) :: a,b
      DOUBLE COMPLEX, DIMENSION(SIZE(a)+SIZE(b)-1) :: x, y, Conv
      DOUBLE PRECISION, DIMENSION(4*(SIZE(a)+SIZE(b)-1)+15) :: wsave
 
      EXTERNAL dcffti, dcfftf, dcfftb

      x = 0.; x(1:SIZE(a)) = a
      y = 0.; y(1:SIZE(b)) = b

      CALL dcffti(SIZE(a)+SIZE(b)-1,wsave)
      CALL dcfftf(SIZE(a)+SIZE(b)-1,x,wsave)
      CALL dcfftf(SIZE(a)+SIZE(b)-1,y,wsave)

      Conv = DHP(x,y)

      CALL dcfftb(SIZE(a)+SIZE(b)-1,Conv,wsave)
      Conv = Conv/(SIZE(a)+SIZE(b)-1)
      END FUNCTION Conv
!
!-----------------------------------------------------------------------
!
      SUBROUTINE Ratint(m,z,f,factor,dl,dr,alpha,hn)
!-----------------------------------------------------------------------
!***BEGIN PROLOGUE Ratint 
!***DATE WRITTEN   960710    (YYMMDD)
!***AUTHOR
!
!   Original MATLAB version by
!
!          Marc Van Barel
!          Dept. Computer Science
!          Katholieke Universiteit Leuven
!          Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
!
!          E-mail: Marc.VanBarel@cs.kuleuven.ac.be
!
!   Translated into FORTRAN 90 by
!
!          Peter Kravanja
!          Dept. Computer Science
!          Katholieke Universiteit Leuven
!          Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
!
!          E-mail: na.pkravanja@na-net.ornl.gov
!
!***DESCRIPTION
!     This routine is called by CHankel.
!     It solves a rational interpolation problem on the unit circle.
!
!***END PROLOGUE Ratint 
!-----------------------------------------------------------------------
!
!  Global variables
!
      INTEGER, INTENT(IN)                :: m
      INTEGER, INTENT(OUT)               :: hn 
      REAL, INTENT(IN)                   :: factor 
      REAL, INTENT(OUT)                  :: alpha 
      COMPLEX, DIMENSION(m)              :: z, f     
      COMPLEX, DIMENSION(m), INTENT(OUT) :: dl, dr
!
!  Local variables
!
      INTEGER                :: i, ipivL, ipivR, ipiv, k
      INTEGER                :: status, flow, degl, degr
      INTEGER                :: deg2l, deg2r
      INTEGER, DIMENSION(15) :: ifac
      REAL                   :: a1, a2, reps, maxpivL, maxpivR
      REAL, DIMENSION(2*m)   :: wa, ch
      COMPLEX                :: mult, hh, hold 
      COMPLEX, DIMENSION(m)  :: L, R, coeffDL, coeffDR
      COMPLEX, DIMENSION(m)  :: tl, tr, coeffTL, coeffTR
!
!  External routines
!
      EXTERNAL cffti1, cfftf1, cfftb1
!
!-----------------------------------------------------------------------
!  status = 1 : both left and right choice are possible
!  status = 2 : only left  choice is possible
!  status = 3 : only right choice is possible
!
!  flow = 1 : the left  column contains the pivot element
!  flow = 2 : the right column contains the pivot element
!-----------------------------------------------------------------------
!
      a1 = VMax(ABS(REAL(f))); a2 = VMax(ABS(AIMAG(f)))
      alpha = MAX(a1,a2)
      f = f/alpha

      L = 1.; R = -f 
      status = 1
      coeffDL = 0.; coeffDR = 0.
      coeffDR(1) = 1.
      degl = 1; degr = 1
      reps = factor*EPSILON(factor)

      DO i = 1, m 
       SELECT CASE (status)
         CASE(1)
           CALL V2Max(ABS(REAL(R(i:m))),ABS(AIMAG(R(i:m))),maxpivR,ipivR)
           ipivR = ipivR + i-1
           CALL V2Max(ABS(REAL(L(i:m))),ABS(AIMAG(L(i:m))),maxpivL,ipivL)
           ipivL = ipivL + i-1
           IF (maxpivR < reps .OR. maxpivL < reps) THEN
             status = 1
             hn = i-1
             RETURN
           END IF
           IF (maxpivR > maxpivL) THEN
             ipiv = ipivR; flow = 2; status = 2
           ELSE
             ipiv = ipivL; flow = 1; status = 3
           END IF
         CASE(2)
           CALL V2Max(ABS(REAL(L(i:m))),ABS(AIMAG(L(i:m))),maxpivL,ipivL)
           ipivL = ipivL + i-1
           IF (maxpivL < reps) THEN
             status = 2
             hn = i-1
             RETURN
           END IF 
           ipiv = ipivL; flow = 1; status = 1
         CASE(3)
           CALL V2Max(ABS(REAL(R(i:m))),ABS(AIMAG(R(i:m))),maxpivR,ipivR)
           ipivR = ipivR + i-1
           IF (maxpivR < reps) THEN
             status = 3
             hn = i-1
             RETURN
           END IF
           ipiv = ipivR; flow = 2; status = 1
       END SELECT

       hold = z(i); z(i) = z(ipiv); z(ipiv) = hold 
       hold = L(i); L(i) = L(ipiv); L(ipiv) = hold 
       hold = R(i); R(i) = R(ipiv); R(ipiv) = hold 

       SELECT CASE(flow)
         CASE(1)
           mult = -R(i)/L(i)
           R(i+1:m) = R(i+1:m) + mult*L(i+1:m)

           coeffDR(1:degl) = coeffDR(1:degl) + mult*coeffDL(1:degl)
           hh = coeffDL(degl)
           coeffDL(2:degl) = coeffDL(1:degl-1) - z(i)*coeffDL(2:degl)
           coeffDL(1) = - coeffDL(1)*z(i)
           coeffDL(degl + 1) = hh
           degl = degl + 1

           DO k = i+1, m
             L(k) = L(k) * (z(k) - z(i)) 
           END DO
         CASE(2)
           mult = -L(i)/R(i) 
           L(i+1:m) = L(i+1:m) + mult*R(i+1:m)

           coeffDL(1:degr) = coeffDL(1:degr) + mult*coeffDR(1:degr)
           hh = coeffDR(degr)
           coeffDR(2:degr) = coeffDR(1:degr-1) - z(i)*coeffDR(2:degr)
           coeffDR(1) = - coeffDR(1)*z(i)
           coeffDR(degr + 1) = hh
           degr = degr + 1

           DO k = i+1, m
             R(k) = R(k) * (z(k) - z(i))
           END DO
       END SELECT

9999   FORMAT (' *** flow = ',I1,', status = ',I1/)
9998   FORMAT (' *** status = ',I1/)
      END DO
      hn = m

      CALL cffti1(m,wa,ifac)
      coeffDL = CONJG(coeffDL) 
      coeffDR = CONJG(coeffDR)
      CALL cfftf1(m,coeffDL,ch,wa,ifac)
      CALL cfftf1(m,coeffDR,ch,wa,ifac)
      dl = CONJG(coeffDL)
      dr = CONJG(coeffDR) 

      END SUBROUTINE Ratint
!
!-----------------------------------------------------------------------
!
      FUNCTION VMax(a)
      REAL, DIMENSION(:) :: a
      REAL VMax
      INTEGER k

      VMax = a(1)
      DO k = 2, SIZE(a)
        IF (a(k) > VMax) VMax = a(k)
      END DO 

      END FUNCTION VMax
!
!-----------------------------------------------------------------------
!
      SUBROUTINE V2Max(a,b,maximum,where)
      REAL, DIMENSION(:), INTENT(IN) :: a
      REAL, DIMENSION(SIZE(a)), INTENT(IN) :: b
      REAL, INTENT(OUT) :: maximum
      REAL m
      INTEGER, INTENT(OUT) :: where 
      INTEGER k

      maximum = MAX(a(1),b(1))
      where = 1
      DO k = 2, SIZE(a)
        m = MAX(a(k),b(k))
        IF (m > maximum) THEN
          maximum = m
          where = k
        END IF 
      END DO 

      END SUBROUTINE V2Max

END MODULE Hankel_Single
